import os
import time 
import streamlit as st
import streamlit.components.v1 as components
from st_screen_stats import IS_RELEASE


if not IS_RELEASE:
    _st_window_query_size = components.declare_component(

        "st_screen_data",

        url="http://localhost:3001",
    )
else:
    # build directory:
    parent_dir = os.path.dirname(os.path.abspath(__file__))
    build_dir = os.path.join(parent_dir, "frontend/build")
    _st_window_query_size = components.declare_component("st_window_query_size", path=build_dir)


class WindowQuerySize:
    """
    Query screen using window.matchMedia() which works similarly to css' `@media () {}` query.

    Params on init:
        - pause [int]: time to first pause the component to give it time to mount/load. Will only be implemented once when the app first loads. (uses st.session_state - parameter named per method)

    ### Methods:
        - mediaQuery()
        - mediaQueryT()
    """

    def __init__(self, pause:int=1.5) -> None:
        """
            params:
                - pause[int]: period of time to pause when component first mounts/load before data is loaded.
        """
        self.pause = pause


    def mediaQuery(self, mediaMatchQ:str=None, default=None, key=None):
        """
        Boolean result after query screen window (browser tab/iframe/lower window).
        Args:
            - mediaMatchQ: string query. 
                Example: "(max-width: 700px)" which will return boolean result `{status:True}` if the window size is lower or `{status:False}` if window size is greater.

        """

        if mediaMatchQ == None or type(mediaMatchQ) != str:
            return

        value = _st_window_query_size(windowType="window", mediaMatchQ=mediaMatchQ, key=key, default=default)
        if self.pause != None and (type(self.pause) == int or type(self.pause) == float):
            if f"firstRunComponent_{key}" not in st.session_state:
                st.session_state[f"firstRunComponent_{key}"] = False
            
            if not st.session_state[f"firstRunComponent_{key}"]:
                time.sleep(self.pause)
                st.session_state[f"firstRunComponent_{key}"] = True

        return value

    def mediaQueryT(self, mediaMatchQ:str=None, default=None, key=None):
        """
        Boolean result after query screen size of browser window.
        Args:
            - mediaMatchQ: string query. 
                Example: "(max-width: 700px)" which will return boolean result `{status:True}` if the window size is lower or `{status:False}` if window size is greater.

        """

        if mediaMatchQ == None or type(mediaMatchQ) != str:
            return

        value = _st_window_query_size(windowType="windowTop", mediaMatchQ=mediaMatchQ, key=key, default=default)
        if self.pause != None and (type(self.pause) == int or type(self.pause) == float):
            if f"firstRunComponent_{key}" not in st.session_state:
                st.session_state[f"firstRunComponent_{key}"] = False
            
            if not st.session_state[f"firstRunComponent_{key}"]:
                time.sleep(self.pause)
                st.session_state[f"firstRunComponent_{key}"] = True

        return value

class WindowQueryHelper:

    """
        Simplifies creating a query for the top window (main browser window) and window (tab window/iframe and other 'lower' windows)
        Params on init:
        - pause [int]: time to first pause the component to give it time to mount/load. Will only be implemented once when the app first loads. (uses st.session_state - parameter named per method)
        
        ### Methods:
            - minimum_window_size()
            - maximum_window_size()
            - minimum_window_size_top()
            - maximum_window_size_top()
            - window_range_width() 
            - window_top_range_width()

    """

    def __init__(self, pause:int=1.2) -> None:
        self.pause = pause

    def minimum_window_size(self, min_width:int=None, key="min_width_window", default=None):

        """
            Returns a boolean (True) if the width of the screen window (browser tab/iframe/lower window) is greater than `min_width` parameter. False if its lower.
        """
        if min_width == None or type(min_width) != int:
            return
        
        query_result = f"(min-width: {min_width}px)"

        value = _st_window_query_size(windowType="window", mediaMatchQ=query_result, key=key, default=default)
        if self.pause != None and (type(self.pause) == int or type(self.pause) == float):
            if f"firstRunComponent_{key}" not in st.session_state:
                st.session_state[f"firstRunComponent_{key}"] = False
            
            if not st.session_state[f"firstRunComponent_{key}"]:
                time.sleep(self.pause)
                st.session_state[f"firstRunComponent_{key}"] = True

        return value
    
    def maximum_window_size(self, max_width:int=None, key="max_width_window", default=None):

        """
            Returns a boolean (True) if the width of the screen window (browser tab/iframe/lower window) is lower than `max_width` parameter. False if its greater.
        """
        if max_width == None or type(max_width) != int:
            return
        
        query_result = f"(max-width: {max_width}px)"

        value = _st_window_query_size(windowType="window", mediaMatchQ=query_result, key=key, default=default)
        if self.pause != None and (type(self.pause) == int or type(self.pause) == float):
            if f"firstRunComponent_{key}" not in st.session_state:
                st.session_state[f"firstRunComponent_{key}"] = False
            
            if not st.session_state[f"firstRunComponent_{key}"]:
                time.sleep(self.pause)
                st.session_state[f"firstRunComponent_{key}"] = True
        
        return value
    
    def minimum_window_size_top(self, min_width:int=None, key="min_width_window", default=None):

        """
            Returns a boolean (True) if the width of the screen window (browser window) is greater than `min_width` parameter. False if its lower.
        """
        if min_width == None or type(min_width) != int:
            return
        
        query_result = f"(min-width: {min_width}px)"

        value = _st_window_query_size(windowType="windowTop", mediaMatchQ=query_result, key=key, default=default)
        if self.pause != None and (type(self.pause) == int or type(self.pause) == float):
            if f"firstRunComponent_{key}" not in st.session_state:
                st.session_state[f"firstRunComponent_{key}"] = False
            
            if not st.session_state[f"firstRunComponent_{key}"]:
                time.sleep(self.pause)
                st.session_state[f"firstRunComponent_{key}"] = True

        return value
    
    def maximum_window_size_top(self, max_width:int=None, key="max_width_window", default=None):

        """
            Returns a boolean (True) if the width of the screen window (browser window) is lower than `max_width` parameter. False if its greater.
        """
        if max_width == None or type(max_width) != int:
            return
        
        query_result = f"(max-width: {max_width}px)"

        value = _st_window_query_size(windowType="windowTop", mediaMatchQ=query_result, key=key, default=default)
        if self.pause != None and (type(self.pause) == int or type(self.pause) == float):
            if f"firstRunComponent_{key}" not in st.session_state:
                st.session_state[f"firstRunComponent_{key}"] = False
            
            if not st.session_state[f"firstRunComponent_{key}"]:
                time.sleep(self.pause)
                st.session_state[f"firstRunComponent_{key}"] = True

        return value
    
    def window_range_width(self, min_width:int=None, max_width:int=None, default=None, key="window_min_max_range"):
        
        """
            Returns boolean (True) if width of window (browser tab/iframe/lower window) is within the range of `min_width` and `max_width` specified. If not, it returns False.
        """

        if min_width == None or max_width == None or type(min_width) != int or type(max_width) != int:
            return
        
        query_result = f'(min-width: {min_width}px) and (max-width: {max_width}px)'

        value = _st_window_query_size(windowType="window", mediaMatchQ=query_result, key=key, default=default)
        if self.pause != None and (type(self.pause) == int or type(self.pause) == float):
            if f"firstRunComponent_{key}" not in st.session_state:
                st.session_state[f"firstRunComponent_{key}"] = False
            
            if not st.session_state[f"firstRunComponent_{key}"]:
                time.sleep(self.pause)
                st.session_state[f"firstRunComponent_{key}"] = True

        return value
    
    def window_top_range_width(self, min_width:int=None, max_width:int=None, default=None, key="window_min_max_range"):
        
        """
            Returns boolean (True) if width of window (browser window) is within the range of `min_width` and `max_width` specified. If not, it returns False.
        """

        if min_width == None or max_width == None or type(min_width) != int or type(max_width) != int:
            return
        
        query_result = f'(min-width: {min_width}px) and (max-width: {max_width}px)'

        value = _st_window_query_size(windowType="windowTop", mediaMatchQ=query_result, key=key, default=default)
        if self.pause != None and (type(self.pause) == int or type(self.pause) == float):
            if f"firstRunComponent_{key}" not in st.session_state:
                st.session_state[f"firstRunComponent_{key}"] = False
            
            if not st.session_state[f"firstRunComponent_{key}"]:
                time.sleep(self.pause)
                st.session_state[f"firstRunComponent_{key}"] = True

        return value
    



import time
import streamlit as st
# from ScreenData.__init__ import ScreenData

from st_screen_stats import ScreenData, StreamlitNativeWidgetScreen, WindowQuerySize, WindowScreenRange # ScreenData, StreamlitNativeWidgetScreen

st.set_page_config(layout="wide")

screenD = ScreenData()
screen_d = screenD.st_screen_data_window()
st.write(screen_d)

# rangeWindow = WindowScreenRange()
# bool_res_range = rangeWindow.WidthRange(upperRange=700, lowerRange=400)
# st.write(bool_res_range)

# size_r = WindowQuerySize()
# val_ = size_r.mediaQuery(mediaMatchQ="(min-width: 700px)")
# st.write(val_)

# st.subheader("Component with constant args")



# screen_d = screenD.st_screen_data_window(default="Hiya!")  #st_screen_data_window_top()
# time.sleep(2)

# st.write(screen_d)


# st.subheader("native widget method")
# screenDN = StreamlitNativeWidgetScreen()
# screenDN.st_screen_data_window_top()
# stats_ = screenDN.get_window_screen_stats(key="get_item")
# st.write(stats_)

# screenRange = SizeRange()
# val_ = screenRange.WidthRange(lowerRange=400, upperRange=500)
# st.write(val_)

# upperScreenWidth = 1000

js_el = f'''
            <script>

                let testD = window.matchMedia("(max-width: 700px)")
            
                function test (e) {{
                
                    let val_ = window.matchMedia("(max-width: 700px)").matches
                    console.log(val_)
                
               
                }}
                
                
                testD.addEventListener("change", test)
                //window.removeEventListener("resize", test)
                
            </script>

        '''
# st.components.v1.html(js_el)

# # my_component()





from __init__ import WindowQuerySize, WindowQueryHelper
# from st_screen_stats import WindowQuerySize, WindowQueryHelper
import streamlit as st

st.set_page_config(layout="wide")

screenSizeInit = WindowQuerySize()
result = screenSizeInit.mediaQueryT("(max-width: 700px)")
st.write(result)

screenSizeInitAlt = WindowQueryHelper(pause=1.5)
# test_ = screenSizeInitAlt.maximum_window_size(max_width=800, default={"status":False}, key="window_1")
# st.write("max_width",test_)
test_ = screenSizeInitAlt.minimum_window_size(min_width=800, key="window_2", default={}) # default={"status":False}
st.write("min_width",test_)

if test_["status"] == False:
    st.write("Hiya falsy")
else:
    st.write("Hiya Truthy")

# test_ = screenSizeInitAlt.maximum_window_size_top(max_width=800, key="window_top_1")
# st.write("max_width_top",test_)
# test_ = screenSizeInitAlt.minimum_window_size_top(min_width=800, key="window_top_2")
# st.write("min_width_top",test_)

# testing_ = screenSizeInitAlt.window_top_range_width(min_width=1000, max_width=1100)
# st.write("range", testing_)


# # width_threshold = "max"
# # value_ = 700

# # result = f'"({width_threshold}-width: {value_}px)"'

# # st.write(result)

# # width_threshold = "min"
# # value_2 = 900
# # range_result = f'"({width_threshold}-width: {value_}px) and ({width_threshold}-width: {value_2}px)"'

# # st.write(range_result)

# # def single_width_query(measurement="max", threshold=700):

# #     result = f"({measurement}-width: {threshold}px)"

# #     return result

# # st.write(single_width_query())


# # def range_width_query(min_threshold=500, max_threshold=1000):

# #     result = f'"(min-width: {min_threshold}px) and (max-width: {max_threshold}px)"'

# #     return result

# # st.write(range_width_query())



